@@ -15,7 +15,7 @@ static const char rcsid[] = "#(@) $Id$";
 /*  ENCODE  --	Encode binary file into base64.  */
 #include <stdlib.h>
 #include <ctype.h>
-#include <limits.h>
+#include <php.h>
 
 #include "base64.h"
 
@@ -24,7 +24,7 @@ static unsigned char dtable[512];
 void buffer_new(struct buffer_st *b)
 {
   b->length = 512;
-  b->data = malloc(sizeof(char)*(b->length));
+  b->data = emalloc(sizeof(char)*(b->length));
   b->data[0] = 0;
   b->ptr = b->data;
   b->offset = 0;
@@ -39,14 +39,14 @@ void buffer_add(struct buffer_st *b, char c)
   b->offset++;
   if (b->offset == b->length) {
     b->length += 512;
-    b->data = realloc(b->data, b->length);
+    b->data = erealloc(b->data, b->length);
     b->ptr = b->data + b->offset;
   }
 }
 
 void buffer_delete(struct buffer_st *b)
 {
-  free(b->data);
+  efree(b->data);
   b->length = 0;
   b->offset = 0;
   b->ptr = NULL;
@@ -57,9 +57,6 @@ void base64_encode_xmlrpc(struct buffer_st *b, const char *source, int length)
 {
   int i, hiteof = 0;
   int offset = 0;
-  int olen;
-
-  olen = 0;
 
   buffer_new(b);
 
@@ -77,7 +74,7 @@ void base64_encode_xmlrpc(struct buffer_st *b, const char *source, int length)
 
   while (!hiteof) {
     unsigned char igroup[3], ogroup[4];
-	int c, n;
+    int c, n;
 
     igroup[0] = igroup[1] = igroup[2] = 0;
     for (n = 0; n < 3; n++) {
@@ -169,7 +166,7 @@ void base64_decode_xmlrpc(struct buffer_st *bfr, const char *source, int length)
 		return;
 	    }
 
-	    if (dtable[(unsigned char)c] & 0x80) {
+	    if (dtable[c] & 0x80) {
 	      /*
 	      fprintf(stderr, "Offset %i length %i\n", offset, length);
 	      fprintf(stderr, "character '%c:%x:%c' in input file.\n", c, c, dtable[c]);
