@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -532,7 +532,7 @@ PHPDBG_API int phpdbg_parse_variable_with_arg(char *input, size_t len, HashTable
 			last_index[index_len] = 0;
 			if (!(zv = zend_symtable_str_find(parent, last_index, index_len))) {
 				if (!silent) {
-					phpdbg_error("variable", "type=\"undefined\" variable=\"%.*s\"", "%.*s is undefined", (int) i, input);
+					phpdbg_error("variable", "type=\"undefined\" variable=\"%.*s\"", "%.*s is undefined", (int) input[i] == ']' ? i + 1 : i, input);
 				}
 				return FAILURE;
 			}
@@ -687,10 +687,12 @@ PHPDBG_API void phpdbg_xml_var_dump(zval *zv) {
 				break;
 			case IS_ARRAY:
 				myht = Z_ARRVAL_P(zv);
-				if (ZEND_HASH_APPLY_PROTECTION(myht) && ++myht->u.v.nApplyCount > 1) {
-					phpdbg_xml("<recursion />");
-					--myht->u.v.nApplyCount;
-					break;
+				if (!(GC_FLAGS(myht) & GC_IMMUTABLE)) {
+					if (GC_IS_RECURSIVE(myht)) {
+						phpdbg_xml("<recursion />");
+						break;
+					}
+					GC_PROTECT_RECURSION(myht);
 				}
 				phpdbg_xml("<array refstatus=\"%s\" num=\"%d\">", COMMON, zend_hash_num_elements(myht));
 				element_dump_func = phpdbg_xml_array_element_dump;
@@ -698,9 +700,8 @@ PHPDBG_API void phpdbg_xml_var_dump(zval *zv) {
 				goto head_done;
 			case IS_OBJECT:
 				myht = Z_OBJDEBUG_P(zv, is_temp);
-				if (myht && ++myht->u.v.nApplyCount > 1) {
+				if (myht && GC_IS_RECURSIVE(myht)) {
 					phpdbg_xml("<recursion />");
-					--myht->u.v.nApplyCount;
 					break;
 				}
 
@@ -715,7 +716,7 @@ head_done:
 						element_dump_func(val, key, num);
 					} ZEND_HASH_FOREACH_END();
 					zend_hash_apply_with_arguments(myht, (apply_func_args_t) element_dump_func, 0);
-					--myht->u.v.nApplyCount;
+					GC_UNPROTECT_RECURSION(myht);
 					if (is_temp) {
 						zend_hash_destroy(myht);
 						efree(myht);
@@ -759,21 +760,25 @@ PHPDBG_API zend_bool phpdbg_check_caught_ex(zend_execute_data *execute_data, zen
 				return 1;
 			}
 
-			do {
+			cur = &op_array->opcodes[catch];
+			while (1) {
 				zend_class_entry *ce;
-				cur = &op_array->opcodes[catch];
 
-				if (!(ce = CACHED_PTR(Z_CACHE_SLOT_P(EX_CONSTANT(cur->op1))))) {
-					ce = zend_fetch_class_by_name(Z_STR_P(EX_CONSTANT(cur->op1)), EX_CONSTANT(cur->op1) + 1, ZEND_FETCH_CLASS_NO_AUTOLOAD);
-					CACHE_PTR(Z_CACHE_SLOT_P(EX_CONSTANT(cur->op1)), ce);
+				if (!(ce = CACHED_PTR(cur->extended_value & ~ZEND_LAST_CATCH))) {
+					ce = zend_fetch_class_by_name(Z_STR_P(RT_CONSTANT(cur, cur->op1)), RT_CONSTANT(cur, cur->op1) + 1, ZEND_FETCH_CLASS_NO_AUTOLOAD);
+					CACHE_PTR(cur->extended_value & ~ZEND_LAST_CATCH, ce);
 				}
 
 				if (ce == exception->ce || (ce && instanceof_function(exception->ce, ce))) {
 					return 1;
 				}
 
-				catch = cur->extended_value;
-			} while (!cur->result.num);
+				if (cur->extended_value & ZEND_LAST_CATCH) {
+					return 0;
+				}
+
+				cur = OP_JMP_ADDR(cur, cur->op2);
+			}
 
 			return 0;
 		}
@@ -804,10 +809,22 @@ char *phpdbg_short_zval_print(zval *zv, int maxlen) /* {{{ */
 			break;
 		case IS_DOUBLE:
 			spprintf(&decode, 0, "%.*G", 14, Z_DVAL_P(zv));
+
+			/* Make sure it looks like a float */
+			if (zend_finite(Z_DVAL_P(zv)) && !strchr(decode, '.')) {
+				size_t len = strlen(decode);
+				char *decode2 = emalloc(len + strlen(".0") + 1);
+				memcpy(decode2, decode, len);
+				decode2[len] = '.';
+				decode2[len+1] = '0';
+				decode2[len+2] = '\0';
+				efree(decode);
+				decode = decode2;
+			}
 			break;
 		case IS_STRING: {
 			int i;
-			zend_string *str = php_addcslashes(Z_STR_P(zv), 0, "\\\"\n\t\0", 5);
+			zend_string *str = php_addcslashes(Z_STR_P(zv), "\\\"\n\t\0", 5);
 			for (i = 0; i < ZSTR_LEN(str); i++) {
 				if (ZSTR_VAL(str)[i] < 32) {
 					ZSTR_VAL(str)[i] = ' ';
@@ -831,12 +848,17 @@ char *phpdbg_short_zval_print(zval *zv, int maxlen) /* {{{ */
 				ZSTR_VAL(str), ZSTR_LEN(str) <= maxlen ? 0 : '+');
 			break;
 		}
-		case IS_CONSTANT:
-			decode = estrdup("<constant>");
-			break;
-		case IS_CONSTANT_AST:
-			decode = estrdup("<ast>");
+		case IS_CONSTANT_AST: {
+			zend_ast *ast = Z_ASTVAL_P(zv);
+
+			if (ast->kind == ZEND_AST_CONSTANT
+			 || ast->kind == ZEND_AST_CONSTANT_CLASS) {
+				decode = estrdup("<constant>");
+			} else {
+				decode = estrdup("<ast>");
+			}
 			break;
+		}
 		default:
 			spprintf(&decode, 0, "unknown type: %d", Z_TYPE_P(zv));
 			break;
